import os
import sys
import json
import time
import traceback
from threading import Thread
import datetime as dt

import requests
from flask import Blueprint, request, jsonify
import paho.mqtt.publish as mqtt_publish


blueprint = Blueprint('detection', __name__)


def get_sunset() -> dt.datetime:
    sunset_date = dt.datetime.now().date()
    try:
        IPGEOLOCATION_API_KEY = os.environ['IPGEOLOCATION_API_KEY']
        ipgeolocation_api_response = requests.get(f'https://api.ipgeolocation.io/astronomy?apiKey={IPGEOLOCATION_API_KEY}&location=Winter+Haven,+FL')
        ipgeolocation_api_response.raise_for_status()

        astronomical_json = json.loads(ipgeolocation_api_response.content)
        sunset_time = dt.datetime.strptime(astronomical_json['sunset'], '%H:%M').time()
    except Exception:
        traceback.print_exc()
        sunset_time = dt.time(20, 00, 00)
    finally:
        if sunset_time < dt.datetime.now().time():
            # Sunset has already passed today
            sunset_date += dt.timedelta(days=1)
        return dt.datetime.combine(sunset_date, sunset_time)


def reset_all_camera_detection_at_sunset() -> None:
    while True:
        try:
            # Get names of cameras with detection enabled in configuration
            frigate_api_response = requests.get('http://frigate:5000/api/config')
            frigate_api_response.raise_for_status()
            frigate_camera_config = json.loads(frigate_api_response.content)['cameras']

            sunset = get_sunset()
            print(f'Waiting until {sunset} to reset detection for all cameras...', file=sys.stderr)
            seconds_until_sunset = (sunset - dt.datetime.now()).total_seconds()
            time.sleep(seconds_until_sunset)

            for camera_name in frigate_camera_config:
                camera_enabled = frigate_camera_config[camera_name].get('enabled', True)
                detection_enabled = frigate_camera_config[camera_name].get('detect', {}).get('enabled', True)

                if camera_enabled and detection_enabled:
                    set_camera_detection(camera_name, True)
        except Exception:
            traceback.print_exc()


def set_camera_detection(camera: str, value: bool, delay: int = 0) -> None:
    time.sleep(delay)

    mqtt_auth = {'username': os.environ.get('MQTT_USERNAME'), 'password': os.environ.get('MQTT_PASSWORD')}
    if not all(mqtt_auth.values()):
        mqtt_auth = None

    mqtt_publish.single(f'frigate/{camera}/detect/set', 'ON' if value else 'OFF', hostname='mqtt', port=1883, auth=mqtt_auth)


@blueprint.route('/camera/<string:camera>/detect', methods=['POST'])
def camera_detect_POST(camera):
    if not request.json:
        return jsonify({
            'status': 'failure',
            'description': 'Request body needs to be in JSON format'
        }), 400

    value = request.json.get('value', True)
    set_camera_detection(camera, value)

    duration = request.json.get('duration', 0)
    if duration > 0:
        # Start sleeping thread to revert value after duration
        thread = Thread(target=set_camera_detection, args=(camera, not value, duration * 60))
        thread.start()

    return jsonify({
        'status': 'success'
    }), 200


sunset_reset_thread = Thread(target=reset_all_camera_detection_at_sunset, args=())
sunset_reset_thread.start()
